package net.sf.javaprinciples.resource;

@SuppressWarnings("unchecked")
public class CacheResourceFactory<T> implements ResourceFactory<T>
{
    private ResourceCache resourceCache;
    private ResourceFormatter<T,T> resourceFormatter;
    private ResourceGetService<T> resourceGetService;
    private ResourcePutService<T> resourcePutService;
    private ResourceCreator resourceCreator;
    private String resourceBeanName;

    /**
     * @param identifier Never null
     * @return May be null if not present
     * @throws net.sf.javaprinciples.resource.ResourceException if any problem is encountered.
     */
    @Override
    public Resource<T> getResource(ResourceIdentifier identifier) throws ResourceException
    {
        // If cache is set, see whether it contains our identifier
        if (resourceCache != null)
        {
            synchronized(resourceCache)
            {
                if (resourceCache.containsResource(identifier))
                {
                    return resourceCache.getResource(identifier);
                }
            }
        }

        // If the item was not in the cache and we have no service, we must
        // return null
        if (resourceGetService == null)
        {
            return null;
        }

        // If we have a service, but no cache, return directly from the service
        if (resourceCache == null)
        {
            return fetchResource(identifier);
        }

        // We have a service and a cache, but the value is yet to be loaded,
        // we synchronize here as we only want to do this once.
        synchronized (resourceGetService)
        {
            synchronized(resourceCache)
            {
                // If another thread has already loaded this value, return it
                if (resourceCache.containsResource(identifier))
                {
                    return resourceCache.getResource(identifier);
                }
            }

            // Fetch the resource from the service, place it in the cache and
            // return it
            Resource<T> resource = fetchResource(identifier);
            synchronized(resourceCache)
            {
                resourceCache.putResource(identifier, resource);
            }
            return resource;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ResourceIdentifier putResource(Resource resource) throws ResourceException
    {
        return resourcePutService.putResource(null, resource);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ResourceIdentifier putResource(ResourceIdentifier identifier, Resource resource) throws ResourceException
    {
        return resourcePutService.putResource(identifier, resource);
    }

    private Resource<T> fetchResource(ResourceIdentifier identifier) throws ResourceException
    {
        T content = fetchResourceContent(identifier);
        Resource<T> resource = resourceCreator.createResource(resourceBeanName);
        resource.setContent(content);
        return resource;
    }

    private T fetchResourceContent(ResourceIdentifier identifier) throws ResourceException
    {
        T content = resourceGetService.getResource(identifier);
        if (content == null)
        {
            throw new UnsupportedOperationException("content can not be null");
        }

        if (resourceFormatter != null)
        {
            content = resourceFormatter.formatResource(content);
        }
        return content;
    }

    public void setResourceCache(ResourceCache resourceCache)
    {
        this.resourceCache = resourceCache;
    }

    public ResourceCache getResourceCache()
    {
        return resourceCache;
    }

    public void setResourceFormatter(ResourceFormatter<T,T> resourceFormatter)
    {
        this.resourceFormatter = resourceFormatter;
    }

    public ResourceFormatter<T,T> getResourceFormatter()
    {
        return resourceFormatter;
    }

    public void setResourceGetService(ResourceGetService<T> resourceGetService)
    {
        this.resourceGetService = resourceGetService;
    }

    public void setResourcePutService(ResourcePutService<T> resourcePutService)
    {
        this.resourcePutService = resourcePutService;
    }

    public void setResourceBeanName(String resourceBeanName)
    {
        this.resourceBeanName = resourceBeanName;
    }

    public String getResourceBeanName()
    {
        return resourceBeanName;
    }

    public void setResourceCreator(ResourceCreator resourceCreator)
    {
        this.resourceCreator = resourceCreator;
    }

    public ResourceCreator getResourceCreator()
    {
        return resourceCreator;
    }
}
