package net.sf.javaprinciples.resource;

import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.UUID;

/**
 * Representation of a URN object, with an implied namespace and "urn:"
 * prefix. Internally uses a Java URI object to provide validation, comparable
 * facilities. <br/>
 * describe in BNF the syntax of URNs as: <br/>
 *
 * <PRE>&lt;URN&gt; ::= "urn:" &lt;NID&gt; ":" &lt;NSS&gt;</PRE>
 *
 * <br/>
 * where &lt;NID&gt;is the Namespace Identifier, and &lt;NSS&gt; is the
 * Namespace Specific String.
 *
 * Phrases enclosed in quotes are REQUIRED. The leading "urn:" sequence is
 * case-insensitive. The Namespace ID determines the syntactic interpretation of
 * the Namespace Specific String.
 *
 * @see java.net.URI
 */
public class URN implements Serializable, Comparable<URN>
{
    public static final String SEPARATOR = ":";
    public static final String URN_PREFIX = "urn";
    private static final long serialVersionUID = 1923615176359406509L;
    private URI netURI;
    private String nss;
    private String nid;
    private String namespace;

    /**
     * Construct a URN from a string.
     * @param completeUrn A URN in the form urn:jcc.org:resource:datacapture
     * @throws URISyntaxException If a null URN is passed.
     */
    public URN(String completeUrn) throws URISyntaxException
    {
        if (completeUrn == null)
        {
            throw new URISyntaxException("null", "A URN was not provided.");
        }
        this.netURI = new URI(completeUrn);
        // We cant get the nid and nss from the URI class.

        int index = completeUrn.lastIndexOf(SEPARATOR);
        if (index == -1)
        {
            this.nid = null;
            this.nss = completeUrn;
        }
        else
        {
            this.nid = completeUrn.substring(0, index);

            if (index < (completeUrn.length() - 1))
            {
                this.nss = completeUrn.substring(index + 1);
            }
        }
    }

    /**
     * Construct a URN for a UUID.
     */
    public URN() throws URISyntaxException
    {
        nid = "uuid";
        nss = UUID.randomUUID().toString();

        this.netURI = new URI(URN_PREFIX + SEPARATOR + nid + SEPARATOR + nss);
    }

    /**
     * Constructor for a URN.
     *
     * @param ns base namespace
     * @param nid extra names
     * @param nss identifier
     * @throws URISyntaxException if illegal characters are used.
     * @see {@link java.net.URI}
     */
    public URN(String ns, String nid, String nss) throws URISyntaxException
    {
        if (nid == null)
        {
            throw new URISyntaxException(ns + SEPARATOR + "null"
                    + SEPARATOR, "A namespace identifier was not provided.");
        }
        if (nss == null)
        {
            throw new URISyntaxException(ns + SEPARATOR + nid
                    + SEPARATOR + "null",
                    "A namespace specific string was not provided.");
        }
        this.netURI = new URI(URN_PREFIX, ns + SEPARATOR + nid
                + SEPARATOR + nss, null);
        this.nss = nss;
        this.nid = nid;
        this.namespace = ns;
    }

    /**
     * Static convenience constructor for creating a URN.
     *
     * @param nm A Namespace.
     * @param nid The name identifier i.e. awris:resource:codelist:region.country
     * @param nss The identifier
     * @return URN representation.
     * @throws URISyntaxException exception if the nid or nss is null or illegal URI characters are used.
     */
    public static URN createURN(String nm, String nid, String nss)
            throws URISyntaxException
    {
        return new URN(nm, nid, nss);
    }

    /**
     * Create a safe filename from a URN.
     * The characters : and . dont work well with operating systems.
     * So we replace with -.
     * @return a safe string.
     */
    public String getSafeString()
    {
        return toString().replaceAll("[\\.\\:]", "-");
    }

    /**
     * Convenience method for printing the URN.
     *
     * @return String representation of URN.
     */
    public String toString()
    {
        return netURI.toASCIIString();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int compareTo(URN o)
    {
        return netURI.compareTo(o.netURI);
    }

    public String getNss()
    {
        return nss;
    }

    public String getNid()
    {
        return nid;
    }
}
